import { Component } from '@angular/core';
import {DatePipe, NgForOf, NgIf, NgOptimizedImage, NgStyle} from "@angular/common";
import {FormGroup,NonNullableFormBuilder, ReactiveFormsModule, Validators} from "@angular/forms";
import {NzButtonComponent} from "ng-zorro-antd/button";
import {NzColDirective, NzRowDirective} from "ng-zorro-antd/grid";
import {NzDividerComponent} from "ng-zorro-antd/divider";
import {NzFormControlComponent, NzFormDirective, NzFormItemComponent, NzFormLabelComponent} from "ng-zorro-antd/form";
import {NzIconDirective} from "ng-zorro-antd/icon";
import {NzImageModule} from "ng-zorro-antd/image";
import {NzInputDirective} from "ng-zorro-antd/input";
import {NzModalComponent, NzModalContentDirective, NzModalModule} from "ng-zorro-antd/modal";
import {NzPopconfirmDirective} from "ng-zorro-antd/popconfirm";
import {
    NzTableCellDirective,
    NzTableComponent,
    NzTbodyComponent,
    NzTheadComponent,
    NzThMeasureDirective, NzTrDirective
} from "ng-zorro-antd/table";
import {NzUploadComponent, NzUploadFile} from "ng-zorro-antd/upload";
import {NzOptionComponent, NzSelectComponent} from "ng-zorro-antd/select";
import {NzTreeSelectComponent} from "ng-zorro-antd/tree-select";
import {environment} from "../../../../environments/environment";
import {NzMessageService} from "ng-zorro-antd/message";
import {CommonService} from "../../../services/common/common.service";
import {GoodsService} from "../../../services/goods/goods.service";
import {GoodsTypeData} from "../../../model/goods.interface";
import {DiscountService} from "../../../services/marketing/discount.service";
import {NzDatePickerComponent} from "ng-zorro-antd/date-picker";
import {NzInputNumberComponent} from "ng-zorro-antd/input-number";
import {NzDropDownADirective, NzDropDownDirective, NzDropdownMenuComponent} from "ng-zorro-antd/dropdown";
import {NzMenuDirective, NzMenuItemComponent} from "ng-zorro-antd/menu";
import {MyValidators} from "../../../services/common/validators.service";

@Component({
  selector: 'app-coupon-discount-list',
  standalone: true,
  imports: [
    NgForOf,
    NzButtonComponent,
    NzColDirective,
    NzFormControlComponent,
    NzFormDirective,
    NzFormItemComponent,
    NzFormLabelComponent,
    NzInputDirective,
    NzOptionComponent,
    NzRowDirective,
    NzModalModule,
    NzSelectComponent,
    ReactiveFormsModule,
    NzIconDirective,
    NzModalComponent,
    NzTreeSelectComponent,
    NzUploadComponent,
    NzModalContentDirective,
    NgStyle,
    DatePipe,
    NzDividerComponent,
    NzTableCellDirective,
    NzTableComponent,
    NzTbodyComponent,
    NzThMeasureDirective,
    NzTheadComponent,
    NzTrDirective,
    NzPopconfirmDirective,
    NzImageModule,
    NgOptimizedImage,
    NzDatePickerComponent,
    NzInputNumberComponent,
    NgIf,
    NzDropDownADirective,
    NzDropDownDirective,
    NzDropdownMenuComponent,
    NzMenuDirective,
    NzMenuItemComponent
  ],
  templateUrl: './coupon-discount-list.component.html',
  styleUrl: './coupon-discount-list.component.less'
})
export class CouponDiscountListComponent {
  // 表单页数
  tablePageNum = 1;
  // 表单数据
  tableData: any = {
    total: 0,
    loading: false,
    list: [],
  };
  // 搜索表单
  searchForm: FormGroup;

  // 图片
  imgUrl: NzUploadFile[] = []
  baseUrl = environment.baseUrl;
  imageUrl = environment.imageUrl;

  isVisibleDetail = false;
  isVisibleGive = false;

  // 编辑产品类型表单
  editForm: FormGroup;
  detailForm: FormGroup;
  giveForm: FormGroup;
  // 编辑弹出框
  isVisibleEdit = false;
  isVisibleDetailEdit = false;
  discountPackageId: number  = 0;
  discountPackage: any;
  discountData: any;

  constructor(private fb: NonNullableFormBuilder,
              private msg: NzMessageService,
              private commonService: CommonService,
              private discountService: DiscountService) {
    // 初始化搜索框
    this.searchForm = this.fb.group({
      title: [''],
    });

    let params =  {
      status: 1,
      pageNum: 1,
      pageSize: 100000,
      time: new Date().getTime()
    }

    this.discountService.getListDiscount(params , (data: any)=>{
      if (data['return_code'] == '000000') {
        this.discountData = data['return_data'].list;
      }
    });
    const { required, maxLength, minLength, email, mobile } = MyValidators;
    // 初始化
    this.editForm = this.fb.group({
      title: ['' , [Validators.required]],
      surplusStock: ['' , [Validators.required]],
      effectiveTime: ['' , [Validators.required]],
      id: [null],
    });
    this.detailForm = this.fb.group({
      discountId: ['' , [Validators.required]],
      num: ['' , [Validators.required]],
      discountPackageId: [''],
      id: [null],
    });
    this.giveForm = this.fb.group({
      phone: ['' , [required, mobile]],
      discountPackageId: [''],
    });
    this.getRequest();

  }

  // 查询列表
  public getRequest(reset: boolean = false) {
    this.tableData.loading  = true;
    if (reset) {
      this.tablePageNum = 1;
    }
    this.searchForm.value.pageNum = this.tablePageNum;
    this.searchForm.value.pageSize = 10;
    this.searchForm.value.time = new Date().getTime();

    this.discountService.getDiscountPackageList(this.searchForm.value , (data: any) => {
      if (data['return_code'] == '000000') {
        this.tableData = data['return_data'];
      } else {
        this.msg.error(data['return_msg']);
      }
      this.tableData.loading  = false;
    });
  }


  // 唤醒编辑表单
  showEdit(item?: GoodsTypeData) {
    if (item != null) {
      this.editForm.patchValue(item);
    } else {
      this.editForm.reset();
    }
    this.isVisibleEdit = true;
  }
  // 编辑表单提交
  handleEdit(): void {

    if (this.editForm.valid) {

      this.discountService.editCouponPackageInfo(this.editForm.value , (data: any) => {
        if (data['return_code'] === '000000') {
          this.msg.success("成功！");
          this.isVisibleEdit = false;
          this.getRequest();
        } else {
          this.msg.error(data['return_msg']);
        }
      });
    } else {
      Object.values(this.editForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsDirty();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }

  }

  // 删除
  delete(id: number): void {
    this.discountService.deleteCouponDiscount(id , (data: any) => {
      if (data['return_code'] === '000000') {
        this.msg.success("成功！");
        this.getRequest();
      } else {
        this.msg.error(data['return_msg']);
      }
    });
  }

  // 删除券包详情
  deleteDetail(id: number): void {
    this.discountService.deleteDetail(id , (data: any) => {
      if (data['return_code'] === '000000') {
        this.msg.success("成功！");
        this.getDetailList(this.discountPackageId);
      } else {
        this.msg.error(data['return_msg']);
      }
    });
  }

  // 查询券包详情
  getDetailList(id: number): void {
    this.discountPackageId = id;
    this.isVisibleDetail = true;
    let params =  {
      discountPackageId: id,
      time: new Date().getTime()
    }
    this.discountService.getDetailList(params, (data: any) => {
      if (data['return_code'] === '000000') {
        this.discountPackage = data['return_data'];
      } else {
        this.msg.error(data['return_msg']);
      }
    });
  }

  giveDiscountPackageShow(id: number): void {
    this.discountPackageId = id;
    this.isVisibleGive = true;
  }

  giveDiscountPackage(): void {
    this.giveForm.value.discountPackageId = this.discountPackageId;
    if (this.giveForm.valid) {

      this.discountService.giveDiscountPackage(this.giveForm.value , (data: any) => {
        if (data['return_code'] === '000000') {
          this.msg.success("成功！");
          this.getRequest(false);
          this.isVisibleGive = false;
        } else {
          this.msg.error(data['return_msg']);
        }
      });
    } else {
      Object.values(this.giveForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsDirty();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }
  }

  // 创建详情
  showEditDetail(): void {
    this.detailForm.reset();
    this.isVisibleDetailEdit = true;
  }

  handleDetailEdit(): void {
    this.detailForm.value.discountPackageId = this.discountPackageId;
    if (this.detailForm.valid) {

      this.discountService.insertCouponPackageDetail(this.detailForm.value , (data: any) => {
        if (data['return_code'] === '000000') {
          this.msg.success("成功！");
          this.isVisibleDetailEdit = false;
          this.getDetailList(this.discountPackageId);
        } else {
          this.msg.error(data['return_msg']);
        }
      });
    } else {
      Object.values(this.detailForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsDirty();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }
  }

}
